""" Логирование серверного модуля """

import os
import sys
import logging
from logging.handlers import TimedRotatingFileHandler
from common.variables import LOGGING_LEVEL

# Создаём объект-логер с именем app.server
log = logging.getLogger('app.server')

# Создаём объект форматирования:
_formatter = logging.Formatter("%(asctime)s  %(levelname)-8s  %(module)-10s  %(message)s")
_formatter_stream = logging.Formatter("%(levelname)-8s  %(message)s ")

# Создаём файловый обработчик логирования (можно задать кодировку):
PATH = os.path.dirname(os.path.abspath(__file__))
PATH = os.path.join(PATH, 'server.log')
fh = TimedRotatingFileHandler(PATH, when='midnight', backupCount='3', encoding='utf-8')

# fh.setLevel(logging.DEBUG)
fh.setFormatter(_formatter)

# Вывод критических ошибок
sh = logging.StreamHandler(sys.stderr)
sh.setFormatter(_formatter_stream)
sh.setLevel(logging.ERROR)

# Добавляем в логер новый обработчик событий и устанавливаем уровень логирования
log.addHandler(fh)
log.addHandler(sh)
log.setLevel(LOGGING_LEVEL)

if __name__ == '__main__':
    log.error('Отладочное сообщение')




import logging
LOG = logging.getLogger('app.server')


# Дескриптор для описания порта:
class Port:
    def __set__(self, instance, value):
        # instance - экземпляр класса - <__main__.Server object at 0x000000D582740C50>
        # value - значение переменной self.name - 7777
        if not 1023 < value < 65536:
            LOG.critical(
                f'Попытка запуска сервера с указанием неподходящего порта {value}. Допустимы адреса с 1024 до 65535.')
            exit(1)
        # Если порт прошёл проверку, добавляем его в список атрибутов экземпляра
        instance.__dict__[self.name] = value

    # тут прилетит имя переменной по которой работает дескриптор
    def __set_name__(self, owner, name):
        # owner - класс владелец атребута - <class '__main__.Server'>
        # name - имя переменной, с нашем случае - port
        self.name = name




""" Логирование клиентского модуля """

import os
import sys
import logging
from common.variables import LOGGING_LEVEL

# Создаём объект-логер с именем app.client
log = logging.getLogger('app.client')

# Создаём объект форматирования:
_formatter = logging.Formatter("%(asctime)s  %(levelname)-8s  %(module)-10s  %(message)s")
_formatter_stream = logging.Formatter("%(levelname)-8s  %(message)s ")

# Создаём файловый обработчик логирования (можно задать кодировку):
PATH = os.path.dirname(os.path.abspath(__file__))
PATH = os.path.join(PATH, 'client.log')
fh = logging.FileHandler(PATH, encoding='utf-8')
# fh.setLevel(logging.DEBUG)
fh.setFormatter(_formatter)

# Вывод критических ошибок
sh = logging.StreamHandler(sys.stderr)
sh.setFormatter(_formatter_stream)
sh.setLevel(logging.WARNING)

# Добавляем в логер новый обработчик событий и устанавливаем уровень логирования
log.addHandler(fh)
log.addHandler(sh)
log.setLevel(LOGGING_LEVEL)

if __name__ == '__main__':
    # Создаём потоковый обработчик логирования (по умолчанию sys.stderr):
    stream_hand = logging.StreamHandler()
    # console.setLevel(logging.DEBUG)
    stream_hand.setFormatter(_formatter)
    log.addHandler(stream_hand)
    # В логирование передаем имя текущей функции и имя вызвавшей функции
    log.debug('Отладочное сообщение')


""" Константы проекта """

import logging

# сокет
DEFAULT_PORT = 7777
DEFAULT_IP_ADDRESS = '127.0.0.1'
MAX_CONNECTIONS = 5
MAX_PACKET_LENGTH = 1024
ENCODING = 'utf-8'
CONNECTION_TIMEOUT = 0.5
# Текущий уровень логирования
LOGGING_LEVEL = logging.DEBUG
# Конфигурационный файл сервера:
SERVER_CONFIG = 'server.ini'

# протокол JIM
ACTION = 'action'
TIME = 'time'
USER = 'user'
ACCOUNT_NAME = 'account_name'
SENDER = 'from'
DESTINATION = 'to'
DATA = 'bin'
PUBLIC_KEY = 'pubkey'

PRESENCE = 'presence'
RESPONSE = 'response'
ERROR = 'error'
MESSAGE = 'msg'
MESSAGE_TEXT = 'message'
EXIT = 'exit'
GET_CONTACTS = 'get_contacts'
REMOVE_CONTACT = 'remove'
ADD_CONTACT = 'add'
LIST_INFO = 'data_list'
USERS_REQUEST = 'get_users'
PUBLIC_KEY_REQUEST = 'pubkey_need'


# Словари - ответы:
# 200
RESPONSE_200 = {RESPONSE: 200}
# 202
RESPONSE_202 = {RESPONSE: 202, LIST_INFO: None}
#
RESPONSE_205 = {RESPONSE: 205}
# 400
RESPONSE_400 = {RESPONSE: 400, ERROR: None}
# 511
RESPONSE_511 = {RESPONSE: 511, DATA: None}


from setuptools import setup, find_packages

setup(name="sh_py_mess_server",
      version="0.0.1",
      description="GeekBrains Mess Server. Coursework in a programming language.",
      author="Ivan Ivanov",
      author_email="iv.iv@yandex.ru",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex'],
      scripts=['server/server_run']
      )



""" Утилиты """

import json

from common.variables import MAX_PACKET_LENGTH, ENCODING
from common.decorators import log

@log
def get_message(client):
    """
    Приём сообщения.
    Принимает объект сокета и достает из него байтовое сообщение.
    Возвращает словарь либо ошибку ValueError.
    :param client: объект сокета
    :return dict:
    """

    data_bytes = client.recv(MAX_PACKET_LENGTH)
    if not isinstance(data_bytes, bytes):
        raise ValueError('Получили НЕ байтовые данные')

    data_str = data_bytes.decode(ENCODING)
    if not isinstance(data_str, str):
        raise ValueError('Полученное сообщение не является строкой')

    if data_str == '':
        raise TypeError('Получено пустое сообщение')

    data_dict = json.loads(data_str)
    if not isinstance(data_dict, dict):
        raise ValueError('Аргумент функции должен быть словарём.')

    return data_dict


@log
def send_message(sock, message):
    """
    Передача сообщения.
    Принимает объект сокета и словарь с JIM сообщением, социализирует в json, кодирует в байты и отправляет в сокет.
    :param sock:
    :param dict message:
    :return:
    """

    if not isinstance(message, dict):
        raise TypeError('Аргумент функции должен быть словарём.')

    message_bytes = json.dumps(message).encode(ENCODING)
    sock.send(message_bytes)
